Imports System.Runtime.InteropServices
Imports Microsoft.Win32.SafeHandles

Module Module1
  Public Const FILE_ATTRIBUTE_NORMAL As Short = &H80S
  Public Const FILE_FLAG_NO_BUFFERING As Integer = &H20000000
  Public Const FILE_FLAG_WRITE_THROUGH As Integer = &H80000000

  Public Const PIPE_ACCESS_DUPLEX As Short = &H3S
  Public Const PIPE_READMODE_MESSAGE As Short = &H2S
  Public Const PIPE_TYPE_MESSAGE As Short = &H4S
  Public Const PIPE_WAIT As Short = &H0S

  Public Const INVALID_HANDLE_VALUE As Short = -1

    Friend Structure STORAGE_DEVICE_NUMBER
        Friend DeviceType As Integer
        Friend DeviceNumber As Integer
        Friend PartitionNumber As Integer
    End Structure

    Friend Enum EFileAccess As System.Int32
        ''
        ''  The following are masks for the predefined standard access types
        ''

        DELETE = &H10000
        READ_CONTROL = &H20000
        WRITE_DAC = &H40000
        WRITE_OWNER = &H80000
        SYNCHRONIZE = &H100000

        STANDARD_RIGHTS_REQUIRED = &HF0000
        STANDARD_RIGHTS_READ = READ_CONTROL
        STANDARD_RIGHTS_WRITE = READ_CONTROL
        STANDARD_RIGHTS_EXECUTE = READ_CONTROL
        STANDARD_RIGHTS_ALL = &H1F0000
        SPECIFIC_RIGHTS_ALL = &HFFFF

        ''
        '' AccessSystemAcl access type
        ''

        ACCESS_SYSTEM_SECURITY = &H1000000

        ''
        '' MaximumAllowed access type
        ''

        MAXIMUM_ALLOWED = &H2000000

        ''
        ''  These are the generic rights.
        ''

        GENERIC_READ = &H80000000
        GENERIC_WRITE = &H40000000
        GENERIC_EXECUTE = &H20000000
        GENERIC_ALL = &H10000000
    End Enum

    Friend Enum EFileShare
        FILE_SHARE_NONE = &H0
        FILE_SHARE_READ = &H1
        FILE_SHARE_WRITE = &H2
        FILE_SHARE_DELETE = &H4
    End Enum

    Friend Enum ECreationDisposition
        ''' <summary>
        ''' Creates a new file, only if it does not already exist.
        ''' If the specified file exists, the function fails and the last-error code is set to ERROR_FILE_EXISTS (80).
        ''' If the specified file does not exist and is a valid path to a writable location, a new file is created.
        ''' </summary>
        CREATE_NEW = 1

        ''' <summary>
        ''' Creates a new file, always.
        ''' If the specified file exists and is writable, the function overwrites the file, the function succeeds, and last-error code is set to ERROR_ALREADY_EXISTS (183).
        ''' If the specified file does not exist and is a valid path, a new file is created, the function succeeds, and the last-error code is set to zero.
        ''' For more information, see the Remarks section of this topic.
        ''' </summary>
        CREATE_ALWAYS = 2

        ''' <summary>
        ''' Opens a file or device, only if it exists.
        ''' If the specified file or device does not exist, the function fails and the last-error code is set to ERROR_FILE_NOT_FOUND (2).
        ''' For more information about devices, see the Remarks section.
        ''' </summary>
        OPEN_EXISTING = 3

        ''' <summary>
        ''' Opens a file, always.
        ''' If the specified file exists, the function succeeds and the last-error code is set to ERROR_ALREADY_EXISTS (183).
        ''' If the specified file does not exist and is a valid path to a writable location, the function creates a file and the last-error code is set to zero.
        ''' </summary>
        OPEN_ALWAYS = 4

        ''' <summary>
        ''' Opens a file and truncates it so that its size is zero bytes, only if it exists.
        ''' If the specified file does not exist, the function fails and the last-error code is set to ERROR_FILE_NOT_FOUND (2).
        ''' The calling process must open the file with the GENERIC_WRITE bit set as part of the dwDesiredAccess parameter.
        ''' </summary>
        TRUNCATE_EXISTING = 5
    End Enum

    Friend Enum EFileAttributes
        FILE_ATTRIBUTE_READONLY = &H1
        FILE_ATTRIBUTE_HIDDEN = &H2
        FILE_ATTRIBUTE_SYSTEM = &H4
        FILE_ATTRIBUTE_DIRECTORY = &H10
        FILE_ATTRIBUTE_ARCHIVE = &H20
        FILE_ATTRIBUTE_DEVICE = &H40
        FILE_ATTRIBUTE_NORMAL = &H80
        FILE_ATTRIBUTE_TEMPORARY = &H100
        FILE_ATTRIBUTE_SPARSE_FILE = &H200
        FILE_ATTRIBUTE_REPARSE_POINT = &H400
        FILE_ATTRIBUTE_COMPRESSED = &H800
        FILE_ATTRIBUTE_OFFLINE = &H1000
        FILE_ATTRIBUTE_NOT_CONTENT_INDEXED = &H2000
        FILE_ATTRIBUTE_ENCRYPTED = &H4000
        FILE_ATTRIBUTE_VIRTUAL = &H10000

        'This parameter can also contain combinations of flags (FILE_FLAG_*) 
        FILE_FLAG_BACKUP_SEMANTICS = &H2000000
        FILE_FLAG_DELETE_ON_CLOSE = &H4000000
        FILE_FLAG_NO_BUFFERING = &H2000000
        FILE_FLAG_OPEN_NO_RECALL = &H100000
        FILE_FLAG_OPEN_REPARSE_POINT = &H200000
        FILE_FLAG_OVERLAPPED = &H40000000
        FILE_FLAG_POSIX_SEMANTICS = &H100000
        FILE_FLAG_RANDOM_ACCESS = &H10000000
        FILE_FLAG_SEQUENTIAL_SCAN = &H8000000
        FILE_FLAG_WRITE_THROUGH = &H80000000
    End Enum

  '  <StructLayout(LayoutKind.Sequential, Pack:=1, CharSet:=CharSet.Ansi)> _
  Public Structure SECURITY_ATTRIBUTES
    Public nLength As Integer
    Public lpSecurityDescriptor As Integer
    Public bInheritHandle As Integer
  End Structure

  Public Structure T_OVERLAPPED
    Public internal As Integer
    Public internalHigh As Integer
    Public offset As Integer
    Public offsetHigh As Integer
    Public hEvent As Integer
  End Structure

  Public Declare Function CreateEvent Lib "kernel32" Alias "CreateEventA" _
  (ByRef lpEventAttributes As SECURITY_ATTRIBUTES, ByVal bManualReset As Integer, ByVal bInitialState As Integer, ByVal lpName As String) As Integer

  Declare Function CreateNamedPipe Lib "kernel32" Alias "CreateNamedPipeA" _
  (ByVal lpName As String, ByVal dwOpenMode As Integer, _
  ByVal dwPipeMode As Integer, ByVal nMaxInstances As Integer, _
  ByVal nOutBufferSize As Integer, ByVal nInBufferSize As Integer, _
  ByVal nDefaultTimeOut As Integer, ByVal lpSecurityAttributes As IntPtr _
  ) As Integer

  Declare Function ConnectNamedPipe Lib "kernel32" _
      (ByVal hNamedPipe As Integer, ByVal lpOverlapped As Integer) As Integer

  Declare Function DisconnectNamedPipe Lib "kernel32" _
      (ByVal hNamedPipe As Integer) As Integer

    'Declare Function WriteFile Lib "kernel32" _
    '(ByVal hFile As Integer, ByVal lpBuffer() As Byte, _
    'ByVal nNumberOfBytesToWrite As Integer, ByRef lpNumberOfBytesWritten As Integer, _
    'ByRef lpOverlapped As T_OVERLAPPED _
    ') As Integer
    <DllImport("kernel32.dll")> Friend Function WriteFile( _
    ByVal File As SafeFileHandle, _
    ByVal Buffer As System.Text.StringBuilder, _
    ByVal nNumberOfBytesToWrite As Integer, _
    ByRef NumberOfBytesWritten As Integer, _
    ByRef Overlapped As System.Threading.NativeOverlapped) As SafeFileHandle
    End Function

    <DllImport("kernel32.dll")> Friend Function ReadFile( _
        ByVal File As SafeFileHandle, _
        ByVal Buffer As System.Text.StringBuilder, _
        ByVal NumberOfBytesToRead As Integer, _
        ByRef NumberOfBytesRead As Integer, _
        ByRef Overlapped As System.Threading.NativeOverlapped) As SafeFileHandle
    End Function


    'Declare Function ReadFile Lib "kernel32" _
    '(ByVal hFile As Integer, ByVal lpBuffer() As Byte, _
    'ByVal nNumberOfBytesToRead As Integer, ByRef lpNumberOfBytesRead As Integer, _
    'ByRef lpOverlapped As T_OVERLAPPED _
    ') As Integer

  Declare Function FlushFileBuffers Lib "kernel32" _
      (ByVal hFile As Integer) As Integer

    'Declare Function CloseHandle Lib "kernel32" _
    '    (ByVal hObject As Integer) As Integer
    <DllImport("kernel32.dll", SetLastError:=True)> _
    Public Function CloseHandle(ByVal hObject As SafeFileHandle) As <MarshalAs(UnmanagedType.Bool)> Boolean
    End Function


  Public Declare Function CallNamedPipe Lib "kernel32" Alias "CallNamedPipeA" _
    (ByVal lpNamedPipeName As String, _
    ByRef lpInBuffer() As Byte, _
    ByVal nInBufferSize As Integer, _
    ByRef lpOutBuffer() As Byte, _
    ByVal nOutBufferSize As Integer, _
    ByRef lpBytesRead As Integer, ByVal nTimeOut As Integer) As Integer

    <DllImport("kernel32.dll", SetLastError:=True)> Friend Function PeekNamedPipe( _
      ByVal hNamedPipe As SafeFileHandle, _
      ByVal Buffer As System.Text.StringBuilder, _
      ByVal NumberOfBytesToRead As Integer, _
      ByRef NumberOfBytesRead As Integer, _
      ByRef nBytesAvail As Integer, ByRef BytesLeftThisMessage As Integer) As Boolean
    End Function

    'Public Declare Function PeekNamedPipe Lib "kernel32" (ByVal hNamedPipe As Integer, ByRef lpBufferNullPtr As Integer, ByVal nBufferSize As Integer, _
    '   ByVal lpBytesReadNullPtr As Integer, ByRef TotalBytesAvail As Integer, ByRef lpBytesLeftThisMessageNullPtr As Integer) As Integer

    Public Declare Function WaitNamedPipe Lib "kernel32" Alias "WaitNamedPipeA" (ByVal lpNamedPipeName As String, ByVal nTimeout As Integer) As Integer

    <DllImport("kernel32.dll", SetLastError:=True, CharSet:=System.Runtime.InteropServices.CharSet.Auto)> _
    Friend Function CreateFile(ByVal lpFileName As String, _
    ByVal dwDesiredAccess As EFileAccess, _
    ByVal dwShareMode As EFileShare, _
    ByVal lpSecurityAttributes As IntPtr, _
    ByVal dwCreationDisposition As ECreationDisposition, _
    ByVal dwFlagsAndAttributes As EFileAttributes, _
    ByVal hTemplateFile As IntPtr) As SafeFileHandle
    End Function

    'Public Declare Function CreateFile Lib "kernel32" Alias "CreateFileA" _
    '      (ByVal lpFileName As String, ByVal _
    '      dwDesiredAccess As Integer, ByVal dwShareMode As Integer, _
    '      ByRef lpSecurityAttributes As SECURITY_ATTRIBUTES, _
    '      ByVal dwCreationDisposition As Integer, ByVal dwFlagsAndAttributes As Integer, _
    '      ByVal hTemplateFile As Integer) As Integer

End Module
